@@ -78,7 +78,7 @@ opts_current = new_defaults()
 
 #' @include plot.R
 
-## a list of options attributes for RStudio
+# a list of options attributes for RStudio
 opts_chunk_attr = local({
   opts = lapply(opts_chunk$get(), class)
   opts[opts == 'NULL'] = 'character'
@@ -86,6 +86,7 @@ opts_chunk_attr = local({
   opts$fig.show = list('asis', 'hold', 'animate', 'hide')
   opts$fig.keep = list('high', 'none', 'all', 'first', 'last')
   opts$fig.align = list('default', 'left', 'right', 'center')
+  opts$fig.showtext = 'logical'
   opts$dev = as.list(names(auto_exts))
   opts$fig.ext = as.list(unique(auto_exts))
   opts$external = opts$sanitize = NULL  # hide these two rare options
@@ -133,26 +134,24 @@ set_alias = function(...) {
 #'   \url{http://yihui.name/knitr/options#package_options}
 #' @export
 #' @examples opts_knit$get('verbose'); opts_knit$set(verbose = TRUE)  # change it
-#' \dontrun{
+#' if (interactive()) {
 #' # for unnamed chunks, use 'fig' as the figure prefix
-#' options(knitr.unnamed.chunk.label='fig')
+#' opts_knit$set(unnamed.chunk.label='fig')
 #' knit('001-minimal.Rmd') # from https://github.com/yihui/knitr-examples
 #' }
 opts_knit = new_defaults(list(
-  progress = TRUE, verbose = FALSE, out.format = NULL, width = 75L,
-  base.dir = NULL, base.url = NULL, child.path = '', upload.fun = identity,
-  animation.fun = NULL, global.device = FALSE, eval.after = NULL,
-  concordance = FALSE, tangle = FALSE, child = FALSE,
-  parent = FALSE, documentation = 1L, aliases = NULL, root.dir = NULL,
-  self.contained = TRUE,
-  header = c(highlight = '', tikz = '', framed = ''),
-  unnamed.chunk.label = 'unnamed-chunk'
-))
-## header should not be set by hand unless you know what you are doing
+  progress = TRUE, verbose = FALSE, width = 75L, eval.after = NULL,
+  base.dir = NULL, base.url = NULL, root.dir = NULL, child.path = '',
+  upload.fun = identity, animation.fun = NULL, global.device = FALSE, global.par = FALSE,
+  concordance = FALSE, documentation = 1L, self.contained = TRUE,
+  unnamed.chunk.label = 'unnamed-chunk',
 
-## tangle: whether I'm in tangle mode; child: whether I'm in child
-## document mode; parent: whether I need to add parent preamble to the
-## child output
+  # internal options; users should not touch them
+  out.format = NULL, child = FALSE, parent = FALSE, tangle = FALSE, aliases = NULL,
+  header = c(highlight = '', tikz = '', framed = ''), global.pars = NULL
+))
+# tangle: whether I'm in tangle mode; child: whether I'm in child document mode;
+# parent: whether I need to add parent preamble to the child output
 
 # you may modify these options in options(knitr.package.foo)
 opts_knit_names = c(
