@@ -5,7 +5,7 @@
 #' provides a few useful hooks, which can also serve as examples of how to
 #' define chunk hooks in \pkg{knitr}.
 #'
-#' The function \code{hook_rgl} can be set as a hook in \pkg{knitr} to save
+#' The function \code{hook_rgl()} can be set as a hook in \pkg{knitr} to save
 #' plots produced by the \pkg{rgl} package. According to the chunk option
 #' \code{dev} (graphical device), plots can be save to different formats
 #' (\code{postscript}: \samp{eps}; \code{pdf}: \samp{pdf}; other devices
@@ -13,11 +13,11 @@
 #' according to chunk options \code{fig.width} and \code{fig.height}. Filenames
 #' are derived from chunk labels and the \code{fig.path} option.
 #'
-#' The function \code{hook_webgl} is a wrapper for the
+#' The function \code{hook_webgl()} is a wrapper for the
 #' \code{\link[rgl]{writeWebGL}()} function in the \pkg{rgl} package. It writes
 #' WebGL code to the output to reproduce the \pkg{rgl} scene in a browser.
 #'
-#' The function \code{hook_pdfcrop} can use the program \command{pdfcrop} to
+#' The function \code{hook_pdfcrop()} can use the program \command{pdfcrop} to
 #' crop the extra white margin when the plot format is PDF to make better use of
 #' the space in the output document, otherwise we often have to struggle with
 #' \code{\link[graphics]{par}} to set appropriate margins. Note
@@ -28,7 +28,7 @@
 #' \command{convert} in ImageMagick is used to trim the white margins (call
 #' \command{convert input -trim output}).
 #'
-#' The function \code{hook_optipng} calls the program \command{optipng} to
+#' The function \code{hook_optipng()} calls the program \command{optipng} to
 #' optimize PNG images. Note the chunk option \code{optipng} can be used to
 #' provide additional parameters to the program \command{optipng}, e.g.
 #' \code{optipng = '-o7'}. See \url{http://optipng.sourceforge.net/} for
@@ -39,7 +39,7 @@
 #' plots), we can use the chunk hook \code{hook_plot_custom} to help write code
 #' for graphics output into the output document.
 #'
-#' The hook \code{hook_purl} can be used to write the code chunks to an R
+#' The hook \code{hook_purl()} can be used to write the code chunks to an R
 #' script. It is an alternative approach to \code{\link{purl}}, and can be more
 #' reliable when the code chunks depend on the execution of them (e.g.
 #' \code{\link{read_chunk}()}, or \code{\link{opts_chunk}$set(eval = FALSE)}).
@@ -47,42 +47,52 @@
 #' \code{purl}, i.e. \code{knit_hooks$set(purl = hook_purl)}. When this hook is
 #' enabled, an R script will be written while the input document is being
 #' \code{\link{knit}}. Currently the code chunks that are not R code or have the
-#' chunk option \code{purl=FALSE} are ignored.
+#' chunk option \code{purl=FALSE} are ignored. Please note when the cache is
+#' turned on (the chunk option \code{cache = TRUE}), no chunk hooks will be
+#' executed, hence \code{hook_purl()} will not work, either. To solve this
+#' problem, we need \code{cache = 2} instead of \code{TRUE} (see
+#' \url{http://yihui.name/knitr/demo/cache/} for the meaning of \code{cache =
+#' 2}).
 #' @rdname chunk_hook
 #' @param before,options,envir see references
 #' @references \url{http://yihui.name/knitr/hooks#chunk_hooks}
 #' @seealso \code{\link[rgl]{rgl.snapshot}}, \code{\link[rgl]{rgl.postscript}}
 #' @export
 #' @examples knit_hooks$set(rgl = hook_rgl)
-#' ## then in code chunks, use the option rgl=TRUE
+#' # then in code chunks, use the option rgl=TRUE
 hook_rgl = function(before, options, envir) {
-  library(rgl)
-  ## after a chunk has been evaluated
-  if (before || rgl.cur() == 0) return()  # no active device
+  # after a chunk has been evaluated
+  if (before || rgl::rgl.cur() == 0) return()  # no active device
   name = fig_path('', options)
-  par3d(windowRect = 100 + options$dpi * c(0, 0, options$fig.width, options$fig.height))
+  rgl::par3d(windowRect = 100 + options$dpi * c(0, 0, options$fig.width, options$fig.height))
   Sys.sleep(.05) # need time to respond to window size change
 
-  ## support 3 formats: eps, pdf and png (default)
-  for (dev in options$dev) switch(
-    dev,
-    postscript = rgl.postscript(str_c(name, '.eps'), fmt = 'eps'),
-    pdf = rgl.postscript(str_c(name, '.pdf'), fmt = 'pdf'),
-    rgl.snapshot(str_c(name, '.png'), fmt = 'png')
-  )
+  in_base_dir(save_rgl(name, options$dev))
 
   options$fig.num = 1L  # only one figure in total
   hook_plot_custom(before, options, envir)
 }
+
+save_rgl = function(name, devices) {
+  if (!file_test('-d', dirname(name))) dir.create(dirname(name), recursive = TRUE)
+  # support 3 formats: eps, pdf and png (default)
+  for (dev in devices) switch(
+    dev,
+    postscript = rgl::rgl.postscript(str_c(name, '.eps'), fmt = 'eps'),
+    pdf = rgl::rgl.postscript(str_c(name, '.pdf'), fmt = 'pdf'),
+    rgl::rgl.snapshot(str_c(name, '.png'), fmt = 'png')
+  )
+}
+
 #' @export
 #' @rdname chunk_hook
 hook_pdfcrop = function(before, options, envir) {
-  ## crops plots after a chunk is evaluated and plot files produced
+  # crops plots after a chunk is evaluated and plot files produced
   ext = options$fig.ext
   if (options$dev == 'tikz' && options$external) ext = 'pdf'
   if (before || (fig.num <- options$fig.num) == 0L) return()
   paths = all_figs(options, ext, fig.num)
-  for (f in paths) plot_crop(f)
+  in_base_dir(for (f in paths) plot_crop(f))
 }
 #' @export
 #' @rdname chunk_hook
@@ -97,12 +107,14 @@ hook_optipng = function(before, options, envir) {
   }
   paths = all_figs(options, ext)
 
-  lapply(paths, function(x) {
-    message('optimizing ', x)
-    x = shQuote(x)
-    cmd = paste('optipng', if (is.character(options$optipng)) options$optipng, x)
-    (if (is_windows()) shell else system)(cmd)
-  })
+  in_base_dir(
+    lapply(paths, function(x) {
+      message('optimizing ', x)
+      x = shQuote(x)
+      cmd = paste('optipng', if (is.character(options$optipng)) options$optipng, x)
+      (if (is_windows()) shell else system)(cmd)
+    })
+  )
   return()
 }
 #' @export
@@ -112,35 +124,33 @@ hook_plot_custom = function(before, options, envir){
   if (options$fig.show == 'hide') return() # do not show figures
 
   ext = options$fig.ext %n% dev2ext(options$dev)
-  name = fig_path('', options)
   hook = knit_hooks$get('plot')
 
   n = options$fig.num
   if (n == 0L) n = options$fig.num = 1L # make sure fig.num is at least 1
-  if (n <= 1L) hook(paste(name[1], ext[1], sep = '.'), options) else {
-    res = unlist(lapply(seq_len(n), function(i) {
-      options$fig.cur = i
-      hook(sprintf('%s%s.%s', name, i, ext), reduce_plot_opts(options))
-    }), use.names = FALSE)
-    paste(res, collapse = '')
-  }
+  res = unlist(lapply(seq_len(n), function(i) {
+    options$fig.cur = i
+    hook(fig_path(ext, options, i), reduce_plot_opts(options))
+  }), use.names = FALSE)
+  paste(res, collapse = '')
 }
 #' @export
 #' @rdname chunk_hook
 hook_webgl = function(before, options, envir) {
-  library(rgl)
-  ## after a chunk has been evaluated
-  if (before || rgl.cur() == 0) return()  # no active device
+  # after a chunk has been evaluated
+  if (before || rgl::rgl.cur() == 0) return()  # no active device
   name = tempfile('rgl', '.', '.html'); on.exit(unlink(name))
-  par3d(windowRect = 100 + options$dpi * c(0, 0, options$fig.width, options$fig.height))
+  rgl::par3d(windowRect = 100 + options$dpi * c(0, 0, options$fig.width, options$fig.height))
   Sys.sleep(.05) # need time to respond to window size change
 
   prefix = gsub('[^[:alnum:]]', '_', options$label) # identifier for JS, better be alnum
   prefix = sub('^([^[:alpha:]])', '_\\1', prefix) # should start with letters or _
   writeLines(sprintf(c('%%%sWebGL%%', '<script>%swebGLStart();</script>'), prefix),
              tpl <- tempfile())
-  writeWebGL(dir = dirname(name), filename = name, template = tpl, prefix = prefix,
-             snapshot = FALSE)
+  rgl::writeWebGL(
+    dir = dirname(name), filename = name, template = tpl, prefix = prefix,
+    snapshot = FALSE
+  )
   res = readLines(name)
   res = res[!grepl('^\\s*$', res)] # remove blank lines
   # remove <script src="CanvasMatrix.js" type="text/javascript"></script> (bug #755)
