@@ -23,37 +23,37 @@
 #' @references \url{http://yihui.name/knitr/hooks}
 #' @seealso \code{\link{hook_plot_custom}}
 #' @export
-#' @examples ## this is what happens for a chunk like this
+#' @examples # this is what happens for a chunk like this
 #'
-#' ## <<foo-bar-plot, dev='pdf', fig.align='right'>>=
+#' # <<foo-bar-plot, dev='pdf', fig.align='right'>>=
 #' hook_plot_tex('foo-bar-plot.pdf', opts_chunk$merge(list(fig.align='right')))
 #'
-#' ## <<bar, dev='tikz'>>=
+#' # <<bar, dev='tikz'>>=
 #' hook_plot_tex('bar.tikz', opts_chunk$merge(list(dev='tikz')))
 #'
-#' ## <<foo, dev='pdf', fig.show='animate', interval=.1>>=
+#' # <<foo, dev='pdf', fig.show='animate', interval=.1>>=
 #'
-#' ## 5 plots are generated in this chunk
+#' # 5 plots are generated in this chunk
 #' hook_plot_tex('foo5.pdf', opts_chunk$merge(list(fig.show='animate',interval=.1,fig.cur=5, fig.num=5)))
 hook_plot_tex = function(x, options) {
-  ## This function produces the image inclusion code for LaTeX.
-  ## optionally wrapped in code that resizes it, aligns it, handles it
-  ## as a subfigure, and/or wraps it in a float. Here is a road map of
-  ## the intermediate variables this function fills in (or leaves empty,
-  ## as needed), and an impression of their (possible) contents.
-  ##
-  ##     fig1,                   # \begin{...}[...]
-  ##       sub1,                 #   \subfloat[...]{
-  ##         align1,             #     {\centering
-  ##           resize1,          #       \resizebox{...}{...}{
-  ##             tikz code       #         '\\input{chunkname.tikz}'
-  ##             or animate code #         or '\\animategraphics[size]{1/interval}{chunkname}{1}{fig.num}'
-  ##             or plain code   #         or '\\includegraphics[size]{chunkname}'
-  ##           resize2,          #       }
-  ##         align2,             #     }
-  ##       sub2,                 #   }
-  ##     fig2                    #   \caption[...]{...\label{...}}
-  ##                             # \end{...}  % still fig2
+  # This function produces the image inclusion code for LaTeX.
+  # optionally wrapped in code that resizes it, aligns it, handles it
+  # as a subfigure, and/or wraps it in a float. Here is a road map of
+  # the intermediate variables this function fills in (or leaves empty,
+  # as needed), and an impression of their (possible) contents.
+  #
+  #     fig1,                   # \begin{...}[...]
+  #       sub1,                 #   \subfloat[...]{
+  #         align1,             #     {\centering
+  #           resize1,          #       \resizebox{...}{...}{
+  #             tikz code       #         '\\input{chunkname.tikz}'
+  #             or animate code #         or '\\animategraphics[size]{1/interval}{chunkname}{1}{fig.num}'
+  #             or plain code   #         or '\\includegraphics[size]{chunkname}'
+  #           resize2,          #       }
+  #         align2,             #     }
+  #       sub2,                 #   }
+  #     fig2                    #   \caption[...]{...\label{...}}
+  #                             # \end{...}  % still fig2
 
   rw = options$resize.width
   rh = options$resize.height
@@ -74,22 +74,22 @@ hook_plot_tex = function(x, options) {
   if (!tikz && animate && fig.cur < fig.num) return('')
 
   usesub = length(subcap <- options$fig.subcap) && fig.num > 1
-  ## multiple plots: begin at 1, end at fig.num
+  # multiple plots: begin at 1, end at fig.num
   ai = options$fig.show != 'hold'
 
-  ## TRUE if this picture is standalone or first in set
+  # TRUE if this picture is standalone or first in set
   plot1 = ai || fig.cur <= 1L
-  ## TRUE if this picture is standalone or last in set
+  # TRUE if this picture is standalone or last in set
   plot2 = ai || fig.cur == fig.num
 
-  ## open align code if this picture is standalone/first in set/a subpic
+  # open align code if this picture is standalone/first in set/a subpic
   align1 = if (plot1 || usesub)
     switch(a, left = '\n\n', center = '\n\n{\\centering ', right = '\n\n\\hfill{}', '\n')
-  ## close align code if this picture is standalone/last in set/a subpic
+  # close align code if this picture is standalone/last in set/a subpic
   align2 = if (plot2 || usesub)
     switch(a, left = '\\hfill{}\n\n', center = '\n\n}\n\n', right = '\n\n', '')
 
-  ## figure environment: caption, short caption, label
+  # figure environment: caption, short caption, label
   cap = options$fig.cap
   scap = options$fig.scap
   fig1 = fig2 = ''
@@ -115,8 +115,10 @@ hook_plot_tex = function(x, options) {
     # * place caption with label
     # * close figure environment
     if (plot2) {
-      if (is.null(scap)) scap = str_split(cap, '\\.|;|:')[[1L]][1L]
-      scap = if(is.na(scap)) '' else sprintf('[%s]', scap)
+      if (is.null(scap) && !grepl('[{].*?[:.;].*?[}]', cap)) {
+        scap = strsplit(cap, '[$:.;]')[[1L]][1L]
+      }
+      scap = if (is.null(scap) || is.na(scap)) '' else sprintf('[%s]', scap)
       fig2 = sprintf('\\caption%s{%s\\label{%s}}\n\\end{%s}\n', scap, cap,
                      paste(lab, if (mcap) fig.cur, sep = ''), options$fig.env)
     }
@@ -133,7 +135,7 @@ hook_plot_tex = function(x, options) {
     if (tikz) {
       sprintf('\\input{%s}', x)
     } else if (animate) {
-      ## \animategraphics{} should be inserted only *once*!
+      # \animategraphics{} should be inserted only *once*!
       aniopts = options$aniopts
       aniopts = if (is.na(aniopts)) NULL else gsub(';', ',', aniopts)
       size = paste(c(size, sprintf('%s', aniopts)), collapse = ',')
@@ -160,7 +162,7 @@ hook_plot_tex = function(x, options) {
   size = if (options$size == 'normalsize') '' else sprintf('\\%s', options$size)
   if (!ai) x = sprintf('\\begin{knitrout}%s\n%s\n\\end{knitrout}', size, x)
   if (options$split) {
-    name = fig_path('.tex', options)
+    name = fig_path('.tex', options, NULL)
     if (!file.exists(dirname(name)))
       dir.create(dirname(name))
     cat(x, file = name)
@@ -168,13 +170,13 @@ hook_plot_tex = function(x, options) {
   } else x
 }
 
-## rm empty kframe and verbatim environments
+# rm empty kframe and verbatim environments
 .rm.empty.envir = function(x) {
   x = gsub('\\\\begin\\{(kframe)\\}\\s*\\\\end\\{\\1\\}', '', x)
   gsub('\\\\end\\{(verbatim|alltt)\\}\\s*\\\\begin\\{\\1\\}[\n]?', '', x)
 }
 
-## inline hook for tex
+# inline hook for tex
 .inline.hook.tex = function(x) {
   if (is.numeric(x)) {
     x = format_sci(x, 'latex')
@@ -257,7 +259,7 @@ render_latex = function() {
     error = .color.block('\\bfseries\\color{errorcolor}{', '}'),
     inline = .inline.hook.tex, chunk = .chunk.hook.tex,
     plot = function(x, options) {
-      ## escape plot environments from kframe
+      # escape plot environments from kframe
       paste('\\end{kframe}', hook_plot_tex(x, options), '\n\\begin{kframe}', sep = '')
     }
   )
@@ -269,7 +271,7 @@ render_sweave = function() {
   opts_knit$set(out.format = 'sweave')
   test_latex_pkg('Sweave', file.path(R.home('share'), 'texmf', 'tex', 'latex', 'Sweave.sty'))
   set_header(framed = '', highlight = '\\usepackage{Sweave}')
-  ## wrap source code in the Sinput environment, output in Soutput
+  # wrap source code in the Sinput environment, output in Soutput
   hook.i = function(x, options)
     paste(c('\\begin{Sinput}', hilight_source(x, 'sweave', options), '\\end{Sinput}', ''),
           collapse = '\n')
@@ -293,7 +295,7 @@ render_listings = function() {
   invisible(NULL)
 }
 
-## may add textile, and many other markup languages
+# may add textile, and many other markup languages
 
 #' Some potentially useful document hooks
 #'
