@@ -56,7 +56,8 @@
 #'   file
 #' @param quiet whether to suppress the progress bar and messages
 #' @param envir the environment in which the code chunks are to be evaluated
-#'   (can use \code{\link{new.env}()} to guarantee an empty new environment)
+#'   (for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
+#'   \code{\link{globalenv}()})
 #' @param encoding the encoding of the input file; see \code{\link{file}}
 #' @return The compiled document is written into the output file, and the path
 #'   of the output file is returned, but if the \code{output} path is
@@ -86,7 +87,21 @@
 #'   If the \code{output} argument is a file path, it is strongly recommended to
 #'   be in the current working directory (e.g. \file{foo.tex} instead of
 #'   \file{somewhere/foo.tex}), especially when the output has external
-#'   dependencies such as figure files.
+#'   dependencies such as figure files. If you want to write the output to a
+#'   different directory, you are recommended to set the working directory to
+#'   that directory before you knit a document. For example, if the source
+#'   document is \file{foo.Rmd}, and the expected output is \file{out/foo.md},
+#'   you can do \code{setwd('out/'); knit('../foo.Rmd')}, instead of
+#'   \code{knit('foo.Rmd', 'out/foo.md')}.
+#'
+#'   N.B. There is no guarantee that the R script generated by \code{purl()} can
+#'   reproduce the computation done in \code{knit()}. The \code{knit()} process
+#'   can be fairly complicated (special values for chunk options, custom chunk
+#'   hooks, computing engines besides R, and the \code{envir} argument, etc). If
+#'   you want to reproduce the computation in a report generated by
+#'   \code{knit()}, be sure to use \code{knit()}, instead of merely executing
+#'   the R script generated by \code{purl()}. This seems to be obvious, but some
+#'   people \href{http://bit.ly/SnLi6h}{just do not get it}.
 #' @export
 #' @references Package homepage: \url{http://yihui.name/knitr/}. The \pkg{knitr}
 #'   \href{http://bit.ly/117OLVl}{main manual}: and
@@ -122,15 +137,13 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
     opts_knit$set(progress = opts_knit$get('progress') && !quiet)
   } else {
-    oenvir = .knitEnv$knit_global; .knitEnv$knit_global = envir
-    on.exit({.knitEnv$knit_global = oenvir}, add = TRUE)
     opts_knit$set(output.dir = getwd()) # record working directory in 1st run
     knit_log$restore()
     on.exit(chunk_counter(reset = TRUE), add = TRUE) # restore counter
     adjust_opts_knit()
-    ## turn off fancy quotes, use smaller digits/width, warn immediately
+    # turn off fancy quotes, use smaller width
     oopts = options(
-      useFancyQuotes = FALSE, digits = 4L, width = opts_knit$get('width'),
+      useFancyQuotes = FALSE, width = opts_knit$get('width'),
       knitr.in.progress = TRUE, device = pdf_null
     )
     on.exit(options(oopts), add = TRUE)
@@ -142,6 +155,9 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
                   progress = opts_knit$get('progress') && !quiet
     )
   }
+  # store the evaluation environment and restore on exit
+  oenvir = .knitEnv$knit_global; .knitEnv$knit_global = envir
+  on.exit({.knitEnv$knit_global = oenvir}, add = TRUE)
 
   ext = 'unknown'
   if (in.file) {
@@ -173,7 +189,10 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
       con = file(input2, encoding = encoding); on.exit(close(con), add = TRUE); con
     } else input2, warn = FALSE)
   } else split_lines(text) # make sure each element is one line
-  if (!length(text)) return() # a trivial case: simply and exit
+  if (!length(text)) {
+    if (is.character(output)) file.create(output)
+    return(output) # a trivial case: create an empty file and exit
+  }
   text = native_encode(text)
 
   apat = all_patterns; opat = knit_patterns$get()
@@ -200,7 +219,12 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
   }
 
   if (is.null(out_format())) auto_format(ext)
-  ## change output hooks only if they are not set beforehand
+  # in child mode, strip off the YAML metadata in Markdown if exists
+  if (child_mode() && out_format('markdown') && grepl('^---\\s*$', text[1])) {
+    i = grepl('^---\\s*$', text)
+    if (length(i) >= 2) text = text[-(1:i[2])]
+  }
+  # change output hooks only if they are not set beforehand
   if (identical(knit_hooks$get(names(.default.hooks)), .default.hooks) && !child_mode()) {
     getFromNamespace(paste('render', out_format(), sep = '_'), 'knitr')()
     on.exit(knit_hooks$set(.default.hooks), add = TRUE)
@@ -247,6 +271,7 @@ process_file = function(text, output) {
     pb = txtProgressBar(0, n, char = '.', style = 3)
     on.exit(close(pb), add = TRUE)
   }
+  wd = getwd()
   for (i in 1:n) {
     if (!is.null(.knitEnv$terminate)) {
       res[i] = paste(.knitEnv$terminate, collapse = '\n')
@@ -262,6 +287,7 @@ process_file = function(text, output) {
     res[i] = withCallingHandlers(
       if (tangle) process_tangle(group) else process_group(group),
       error = function(e) {
+        setwd(wd)
         cat(res, sep = '\n', file = output %n% '')
         message(
           'Quitting from lines ', paste(current_lines(i), collapse = '-'),
@@ -290,7 +316,7 @@ auto_out_name = function(input, ext = tolower(file_ext(input))) {
   paste(name, collapse = '')
 }
 
-## decide output format based on file extension
+# determine output format based on file extension
 ext2fmt = c(
   rnw = 'latex', snw = 'latex', tex = 'latex', rtex = 'latex', stex = 'latex',
   htm = 'html', html = 'html', rhtml = 'html', rhtm = 'html',
@@ -319,6 +345,7 @@ auto_format = function(ext) {
 #'   the child document (ignored if not a list); when we use the \code{child}
 #'   option in a parent chunk, the chunk options of the parent chunk will be
 #'   passed to the \code{options} argument here
+#' @inheritParams knit
 #' @return A character string of the content of the compiled child document is
 #'   returned as a character string so it can be written back to the parent
 #'   document directly.
@@ -329,10 +356,10 @@ auto_format = function(ext) {
 #'
 #'   The path of the child document is relative to the parent document.
 #' @export
-#' @examples ## you can write \Sexpr{knit_child('child-doc.Rnw')} in an Rnw file 'main.Rnw' to input results from child-doc.Rnw in main.tex
+#' @examples # you can write \Sexpr{knit_child('child-doc.Rnw')} in an Rnw file 'main.Rnw' to input results from child-doc.Rnw in main.tex
 #'
-#' ## comment out the child doc by \Sexpr{knit_child('child-doc.Rnw', eval = FALSE)}
-knit_child = function(..., options = NULL) {
+#' # comment out the child doc by \Sexpr{knit_child('child-doc.Rnw', eval = FALSE)}
+knit_child = function(..., options = NULL, envir = knit_global()) {
   child = child_mode()
   opts_knit$set(child = TRUE) # yes, in child mode now
   on.exit(opts_knit$set(child = child)) # restore child status
@@ -347,7 +374,7 @@ knit_child = function(..., options = NULL) {
       }, add = TRUE)
     }
   }
-  res = knit(..., tangle = opts_knit$get('tangle'),
+  res = knit(..., tangle = opts_knit$get('tangle'), envir = envir,
              encoding = opts_knit$get('encoding') %n% getOption('encoding'))
   paste(c('', res), collapse = '\n')
 }
@@ -396,6 +423,7 @@ wrap.default = function(x, options) return()
 
 #' @export
 wrap.character = function(x, options) {
+  if (options$results == 'hide') return()
   if (output_asis(x, options)) {
     if (!out_format('latex')) return(x)  # latex output still need a tweak
   } else x = comment_out(x, options$comment)
@@ -414,7 +442,11 @@ wrap.knit_asis = function(x, options) {
   if (length(m)) {
     .knitEnv$meta = c(.knitEnv$meta, m)
   }
-  as.character(x)
+  x = as.character(x)
+  if (!out_format('latex')) return(x)
+  # latex output need the \end{kframe} trick
+  options$results = 'asis'
+  knit_hooks$get('output')(x, options)
 }
 
 #' @export
@@ -438,7 +470,11 @@ msg_wrap = function(message, type, options) {
 
 #' @export
 wrap.warning = function(x, options) {
-  msg_wrap(paste('Warning:', x$message, collapse = '\n'), 'warning', options)
+  call = if (is.null(x$call)) '' else {
+    call = deparse(x$call)[1]
+    if (call == 'eval(expr, envir, enclos)') '' else paste(' in', call)
+  }
+  msg_wrap(sprintf('Warning%s: %s', call, x$message), 'warning', options)
 }
 
 #' @export
@@ -448,18 +484,18 @@ wrap.message = function(x, options) {
 
 #' @export
 wrap.error = function(x, options) {
-  msg_wrap(str_c('Error: ', x$message), 'error', options)
+  msg_wrap(as.character(x), 'error', options)
 }
 
 #' @export
 wrap.recordedplot = function(x, options) {
-  ## figure number sequence for multiple plots
+  # figure number sequence for multiple plots
   fig.cur = plot_counter()
   options$fig.cur = fig.cur # put fig num in options
-  name = fig_path(if(options$fig.num <= 1) '' else fig.cur, options)
-  if (!file.exists(dirname(name)))
+  name = fig_path('', options, number = fig.cur)
+  if (!file_test('-d', dirname(name)))
     dir.create(dirname(name), recursive = TRUE) # automatically creates dir for plots
-  ## vectorize over dev, ext and dpi: save multiple versions of the plot
+  # vectorize over dev, ext and dpi: save multiple versions of the plot
   file = mapply(
     save_plot, width = options$fig.width, height = options$fig.height,
     dev = options$dev, ext = options$fig.ext, dpi = options$dpi,
@@ -519,6 +555,7 @@ knit_print.knit_asis = function(x, ...) x
 #' @rdname knit_print
 #' @export
 normal_print = default_handlers$value
+formals(normal_print) = alist(x = , ... = )
 
 #' Mark an R object with a special class
 #'
