@@ -11,7 +11,7 @@
 #' @author Ramnath Vaidyanathan and Yihui Xie
 #' @seealso \code{\link{knit_theme}}
 #' @export
-#' @examples ## http://www.eclipsecolorthemes.org/?view=theme&id=1
+#' @examples # http://www.eclipsecolorthemes.org/?view=theme&id=1
 #' \dontrun{
 #' opts_knit$set(out.format = 'latex')
 #' (css = eclipse_theme(1))
@@ -19,12 +19,11 @@
 #' knit_theme$set(thm)
 #' }
 eclipse_theme = function(id){
-  library(XML)
   url = 'http://www.eclipsecolorthemes.org/?view=empty&action=download&theme=%s&type=xml'
-  doc = xmlParse(sprintf(url, id))
-  docname = xmlAttrs(xmlRoot(doc), 'colorTheme[@name]')['name']
+  doc = XML::xmlParse(sprintf(url, id))
+  docname = XML::xmlAttrs(XML::xmlRoot(doc), 'colorTheme[@name]')['name']
   docname = make.names(tolower(docname))
-  lst = xmlToList(doc)
+  lst = XML::xmlToList(doc)
 
   css = character(length(css2ecl))
   for (i in seq_along(css2ecl)) {
