@@ -1,6 +1,6 @@
 library(testit)
-# do not write output to console
-op = options(knitr.table.output = FALSE)
+
+kable2 = function(...) as.character(kable(...))
 
 assert(
   'kable() works on data frames/matrices of one row',
@@ -10,28 +10,28 @@ assert(
 m = matrix(1:2, nrow = 1, dimnames = list('a', c('x', 'y')))
 assert(
   'kable() does not discard row names when there is only one row',
-  identical(kable(m), c('|   |  x|  y|', '|:--|--:|--:|', '|a  |  1|  2|'))
+  identical(kable2(m), c('|   |  x|  y|', '|:--|--:|--:|', '|a  |  1|  2|'))
 )
 
 assert(
   'kable() recycles the align argument correctly',
-  identical(kable(m, align = 'c'), c('|   | x | y |', '|:--|:-:|:-:|', '|a  | 1 | 2 |'))
+  identical(kable2(m, align = 'c'), c('|   | x | y |', '|:--|:-:|:-:|', '|a  | 1 | 2 |'))
 )
 
 assert(
   'kable() works on character data frames',
-  identical(kable(data.frame(x = 'a')), c('|x  |', '|:--|', '|a  |'))
+  identical(kable2(data.frame(x = 'a')), c('|x  |', '|:--|', '|a  |'))
 )
 
 assert(
   "kable() works on NA's",
-  identical(kable(data.frame(x=c(NA, FALSE))), c('|x     |', '|:-----|', '|NA    |', '|FALSE |'))
+  identical(kable2(data.frame(x=c(NA, FALSE))), c('|x     |', '|:-----|', '|NA    |', '|FALSE |'))
 )
 
 assert(
   'kable() does not add extra spaces to character columns',
   identical(
-    kable(data.frame(x = c(1.2, 4.87), y = c('fooooo', 'bar')), 'latex'),
+    kable2(data.frame(x = c(1.2, 4.87), y = c('fooooo', 'bar')), 'latex'),
     '
 \\begin{tabular}{r|l}
 \\hline
@@ -45,4 +45,11 @@ x & y\\\\
   )
 )
 
-options(op)
+assert(
+  'kable(digits = vector) works on numeric matrices',
+  identical(
+    kable2(matrix(c(1.1, 1.2, 2.3, 2.4), 2, dimnames = list(NULL, c('a', 'b'))),
+           digits = c(0, 1)),
+    c('|  a|   b|', '|--:|---:|', '|  1| 2.3|', '|  1| 2.4|')
+  )
+)
